#!/usr/bin/perl5

###########################################################################
#
# reg.cgi - Perl ENUM registration system
#
# Copyright (c) 2005  Japan Registry Services Co., LTD. All Rights Reserved.
#
# Author: Kazunori Fujiwara <fujiwara@jprs.co.jp>
#
###########################################################################

use strict;

print "Content-type: text/html\n\n";

my $REQUEST_METHOD = $ENV{'REQUEST_METHOD'};
my $CLIENT = $ENV{'REMOTE_ADDR'};
my $USER = $ENV{'REMOTE_USER'};
my $MODE = '';
my $count = 0;
my $BASEDOMAIN = ".apenum.org";
my $BASENUM = "8882005";
my $CGITELDOMAIN = '';
my $DBBASE='/home/db/';
my $DBTELNO=$DBBASE . "no/";
my $CGIHTMLDIR = "$DBBASE/html";
my $SERIALFILE = "/home/db/serial/serial";
my $ZONEFILE = "/home/db/out/8882005.zone";
my $status = "";
my $req = "/home/db/lock/req";
my %naptr;
my $name = '';
my $additional = '';
my $PARAMETER;
my $CGIRETRYURI = "reg.cgi";
my $CGITOPURI = "/";
my $defaultsipdom = "apricot2005.apenum.org";

$| = 1;

if (! ($USER =~ /^(\d\d\d\d)$/)) {
	&error("authentication failure: $USER");
}

sub zonegenreq
{
	open(F, ">$req") || &error("cannot request zonefile generation and sign");
	print F "zonegen request $$\n";
	close(F);
}

sub debug
{
	my ($val) = @_;
	my $i;

	print "MODE=$MODE;<BR>\n";
	print "CLIENT=$CLIENT;<BR>\n";
	print "USER=$USER;<BR>\n";
	print "name=$name;<BR>\n";
	print "additional=$additional;<BR>\n";
	print "count=$count;<BR>\n";
	print "status=$status;<BR>\n";
	for ($i = 0; $i < $count; $i++) {
		printf("naptr %d = %s%d/%d/%s/%s;<BR>\n", $i,$naptr{"delete",$i} ? "delete ":"", $naptr{"order",$i}, $naptr{"preference",$i},$naptr{"service",$i},$naptr{"uri",$i});
	}
	if ($val ne "") {
		print "$val<BR>\n";
	}
	exit;
}

sub error
{
	my($msg) = @_;
	print "Error occured: $msg<BR>\n";
	exit 0;
}

# for old compatibility
my ($n, $CGIINPUT);
if ($REQUEST_METHOD eq "POST") {
	$n=read( STDIN, $CGIINPUT, $ENV{'CONTENT_LENGTH'} );
	# user=a&passwd=b&logout=1
} elsif ($REQUEST_METHOD eq "GET") {
	#REQUEST_URI = /cgi-bin/auth.pl?user=a&passwd=b
	($n, $CGIINPUT) = split(/\?/, $ENV{'REQUEST_URI'}, 2);
} else {
	&error("Wrong Request-Method $REQUEST_METHOD");
}

my ($lines, $key, $val, $val0, $dummy, $key1, $key2);

foreach $lines (split(/&/, $CGIINPUT)) {
	($key, $val) = split(/=/, $lines, 2);
	$val =~ tr/+/ /;
	$val =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack("C",hex($1))/eg;
	$val0 = $val;
	$val =~ tr/\r\n\"//d;
	if ($key eq 'mode') {
		$MODE = $val;
		$MODE =~ y/A-Z/a-z/;
	} elsif ($key eq 'dummy') {
		$dummy = $val;
	} elsif ($key eq 'name') {
		$name = $val;
	} elsif ($key eq 'additional') {
		$additional = $val;
	} elsif ($key eq 'count') {
		$count = $val;
	} else {
		if ($key =~ /^(\D+)(\d+)$/) {
			$key1 = $1;
			$key2 = $2;
			$naptr{$key1,$key2} = $val;
		} else {
			&error("error: unknown parameter:$key1|$key2|$val|$lines");
		}
	}
}

my $CGITELNO = $BASENUM * 10000 + $USER;

$CGITELDOMAIN = join('.', reverse(split(//, $CGITELNO))) . $BASEDOMAIN;

if ($MODE eq "edit" || $MODE eq "") {
    &read_data;
    &writeedit;
}

if ($MODE eq "enter") {
    #&debug;
    &write_data;
    &read_data;
    &writeedit;
}

# debug
&debug;
exit 0;

# /home/db/no/XXXX
#
# Data format:
#       first line = "name","additional", N
#       N lines: preference,service,uri

sub read_data
{
    my $data;
    my $n;

    $count = 0;
    undef %naptr;
    if (!open(F, "$DBTELNO/$USER")) {
	$name = '';
	$additional = '';
	$count = 1;
	$naptr{"preference",0} = 0;
	$naptr{"service",0} = "sip";
	$naptr{"uri",0} = "sip:$USER\@$defaultsipdom";
	return;
    }
    $data = <F>;
    chomp($data);
    if ($data =~ /^\"(([^\"]|\\\")*)\",\"(([^\"]|\\\")*)\",(\d+)$/) {
	$name = $1;
	$additional = $3;
	$count = $5;
	$n = 0;
        while(<F>) {
	    chomp;
	    if (/^(\d+),([a-zA-Z0-9+-:]+),\"(([^\"]|\\\")*)\"$/) {
		$naptr{"preference",$n} = $1;
		$naptr{"service",$n} = $2;
		$naptr{"uri",$n} = $3;
		$n++;
	    }
	    last if ($n == $count || $n > 10);
	}
    }
    close(F);
}

sub uricheck
{
	my($uri) = @_;
	if (! ($uri =~ /^![^!]+\![^!]+\!$/)) {
		$uri =~ y/\!\(\)\$//d;
		return $uri;
	}
	return $uri;
}

sub write_data
{
	my $out;
	my $i;
	my $n = 0;
	my $uri;
	for ($i = 0; $i <= $count; $i++) {
		$uri = &uricheck($naptr{"uri",$i});
		if (!defined($naptr{"delete",$i}) && $naptr{"service",$i} ne "" && $uri ne "") {
			$out .= $naptr{"preference",$i} . "," . $naptr{"service", $i} . ",\"" . $uri . "\"\n";
			$n++;
		}
	}
	open(F, ">$DBTELNO/$USER") || &error("cannot write $USER");
	$out = "\"$name\",\"$additional\",$n\n" . $out;
	print F $out;
	close(F);
	&zonegenreq;
	$status .= "successfully registered and zone/telephone book generation requested<BR>";
}

sub output
{
	my($file, $parameter) = @_;

	if (!open(F, "$CGIHTMLDIR/$file")) {
		print "Cannot open $file\n";
		exit 0;
	}

	while(<F>) {
		s/CGIRETRYURI/$CGIRETRYURI/;
		s/CGITOPURI/$CGITOPURI/;
		s/CGIUSERNO/$USER/;
		s/CGINAME/$name/;
		s/CGIADDITIONAL/$additional/;
		s/CGIPARAMETER/$PARAMETER/;
		s/CGIMETHOD/$MODE/;
		s/CGITELNO/$CGITELNO/;
		s/CGITELDOMAIN/$CGITELDOMAIN/;
		s/CGIstatus/$status/;
		print;
	}
	exit 0;
}

sub writeedit
{
    my $i;

    $PARAMETER .= "<input type=\"hidden\" maxlength=\"16\" size=\"16\" name=\"count\" value=\"$count\">\n" . 
    "NAPTR Resource Record:<TABLE border=\"0\"><TH nowrap>delete</TH><TH nowrap>preference</TH><TH nowrap>ENUM service</TH><TH nowrap>URI/regexp</TH>";
    for ($i = 0; $i <= $count; $i++) {
	$PARAMETER .=sprintf("<TR>
<TD><input type=\"checkbox\" name=\"delete%d\" value=\"1\"></TD>
<TD><input type=\"text\" size=\"3\" maxlength=\"5\" name=\"preference%d\" value=\"%d\"></TD>
<TD><input type=\"text\" size=\"15\" maxlength=\"40\" name=\"service%d\" value=\"%s\"></TD>
<TD><input type=\"text\" size=\"36\" maxlength=\"80\" name=\"uri%d\" value=\"%s\"></TD>
</TR>"
		, $i,
		$i, $naptr{"preference",$i},
		$i, $naptr{"service",$i},
		$i, $naptr{"uri",$i});
    }
    $PARAMETER .= "</TABLE>\n";

    &output("edit.html");
    exit 0;
}
